#include <XSUser/Python/PyModel.h>

#include <XSContainer.h>
#include <XSstreams.h>
#include <XSFit/Fit/Fit.h>
#include <XSModel/GlobalContainer/DataContainer.h>
#include <XSModel/GlobalContainer/ModelContainer.h>
#include <XSModel/Model/Model.h>
#include <XSModel/Model/Component/Component.h>
#include <XSUtil/Parse/XSparse.h>

PyObject*
_pyXspec_createModel(PyObject *self, PyObject *args)
{
   using namespace XSContainer;
   
   const char *modStr=0;
   if (!PyArg_ParseTuple(args,"z", &modStr))
      return NULL;
   if (XSparse::isBlank(modStr))
   {
      string msg("Error: Model definition string is required.");
      PyErr_SetString(PyExc_Exception, msg.c_str());
      return NULL;
   }

   bool isAdded=false;
   string modName("_DEFAULT");
   size_t sourceNum=1;
   try
   {
      string modString(modStr);
      Model* newModel = new Model(modName,sourceNum,modString);
      models->setFirstOfModelName(newModel);
      models->addToList(newModel);
      isAdded = true;
      // attach responses to the new models. If
      // no responses have been defined, this does nothing.
      models->attachResponses(modName);
      // force computation of the models just created.
      models->setCompute(modName);
         
      // If there is data, new models will be active
      // and calculated through Fit::Update.  Else,
      // do it here.  
      if (!datasets->dataArray().size())
      {
         // Note: for mixing models this can never occur, as it
         // has already been prevented above.
         std::vector<Component*> newComps;
         newModel->bundleComponents(newComps);
         for (size_t i=0; i<newComps.size(); ++i)
         {
            newComps[i]->initializeForFit();
         }
         newModel->calculate();
      }

      // check for mixing model, and initialize the mixing transformation
      // if present. This has to happen after all the models are allocated.
      newModel->initializeMixingTransformation(); 
      
      size_t nGroupsForSource = 1;
      const size_t nShow = nGroupsForSource ? nGroupsForSource : 1;
      if (fit->renormType() != Fit::AUTO)
      {
         // If in Fit::AUTO mode, renormalizing will
         // take place during the Notify call below,
         // so let's not print out the params just yet.
         newModel->printHeading();                               
         tcout << *models->lookup(modName,1);   
         newModel->printMixComp(); 
         tcout << string(72,'_') << '\n' << std::endl;
      }                                            
      models->Notify();
      if (fit->renormType() == Fit::AUTO)
      {
         // OK, now let's print them out.
         tcout << "\nParameters have been automatically renormalized." 
               << std::endl;
         newModel->printHeading();                               
         tcout << *models->lookup(modName,1);   
         newModel->printMixComp(); 
         tcout << string(72,'_') << '\n' << std::endl; 
      }
   }
   catch (...)
   {
      if (isAdded) 
         models->remove(modName);
      string msg("Error occurred while creating new model.");
      PyErr_SetString(PyExc_Exception, msg.c_str());
      return NULL;
   }
   
   return Py_BuildValue("i",0);
}
